package org.codehaus.plexus.interpolation.fixed;

/*
 * Copyright 2001-2008 Codehaus Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

public abstract class AbstractDelegatingValueSource
    implements FixedValueSource
{
    
    private final FixedValueSource delegate;

    protected AbstractDelegatingValueSource( FixedValueSource delegate )
    {
        if ( delegate == null )
        {
            throw new NullPointerException( "Delegate ValueSource cannot be null." );
        }
        
        this.delegate = delegate;
    }
    
    protected FixedValueSource getDelegate()
    {
        return delegate;
    }

    public Object getValue( String expression, InterpolationState interpolationState ){
        return getDelegate().getValue( expression, interpolationState );
    }
}
